@@ -176,6 +176,10 @@ args_dict = {
         "--disk path=%(EXISTIMG1)s,device=cdrom --disk path=%(EXISTIMG1)s,device=cdrom",
         # Dir with a floppy dev
         "--disk %(DIR)s,device=floppy",
+        # Driver name and type options
+        "--disk %(EXISTIMG1)s,driver_name=qemu,driver_type=qcow2",
+        # Unknown driver name and type options
+        "--disk %(EXISTIMG1)s,driver_name=foobar,driver_type=foobaz",
       ],
 
       "invalid": [
@@ -242,6 +246,14 @@ args_dict = {
         "--paravirt --location %s --os-variant none" % treedir,
         # URL install with manual os-variant
         "--hvm --location %s --os-variant fedora12" % treedir,
+        # Boot menu
+        "--hvm --pxe --boot menu=on",
+        # Kernel params
+        """--hvm --pxe --boot kernel=/tmp/foo1.img,initrd=/tmp/foo2.img,kernel_args="ro quiet console=/dev/ttyS0" """,
+        # Boot order
+        "--hvm --pxe --boot cdrom,fd,hd,network,menu=off",
+        # Boot w/o other install option
+        "--hvm --boot network,hd,menu=on",
       ],
 
       "invalid": [
@@ -257,6 +269,10 @@ args_dict = {
         "--livecd",
         # Bogus --os-variant
         "--hvm --pxe --os-variant farrrrrrrge"
+        # Boot menu w/ bogus value
+        "--hvm --pxe --boot menu=foobar",
+        # cdrom fail w/ extra-args
+        "--hvm --cdrom %(EXISTIMG1)s --extra-args console=ttyS0",
       ],
      }, # category "install"
 
@@ -303,6 +319,16 @@ args_dict = {
         "--parallel unix,path=/tmp/foo-socket",
         # TCP w/ telnet
         "--serial tcp,host=:1234,protocol=telnet",
+        # --channel guestfwd
+        "--channel pty,target_type=guestfwd,target_address=127.0.0.1:10000",
+        # --channel virtio
+        "--channel pty,target_type=virtio,name=org.linux-kvm.port1",
+        # --channel virtio without name=
+        "--channel pty,target_type=virtio",
+        # --console virtio
+        "--console pty,target_type=virtio",
+        # --console xen
+        "--console pty,target_type=xen",
      ],
      "invalid" : [
         # Bogus device type
@@ -313,6 +339,10 @@ args_dict = {
         "--serial null,path=/tmp/foo",
         # Nonexistent argument
         "--serial udp,host=:1234,frob=baz",
+        # --channel guestfwd without target_address
+        "--channel pty,target_type=guestfwd",
+        # --console unknown type
+        "--console pty,target_type=abcd",
      ],
 
      }, # category 'char'
@@ -354,6 +384,8 @@ args_dict = {
         "--hvm --import --disk path=virt-install",
         # PV Import install
         "--paravirt --import --disk path=virt-install",
+        # Import a floppy disk
+        "--hvm --import --disk path=virt-install,device=floppy",
         # --autostart flag
         "--hvm --nodisks --pxe --autostart",
         # --description
@@ -372,6 +404,10 @@ args_dict = {
         "--hvm --nodisks --pxe --security type=dynamic",
         # --security implicit static
         "--hvm --nodisks --pxe --security label=foobar.label",
+        # --security static with commas 1
+        "--hvm --nodisks --pxe --security label=foobar.label,a1,z2,b3,type=static",
+        # --security static with commas 2
+        "--hvm --nodisks --pxe --security label=foobar.label,a1,z2,b3",
       ],
 
       "invalid": [
@@ -423,7 +459,7 @@ args_dict = {
         # Mixing bridge and network
         "--network user --bridge foo0",
         # Colliding macaddr
-        "--mac 11:22:33:44:55:66",
+        "--mac 11:22:33:12:34:AB",
       ],
 
      }, # category "network"
@@ -540,11 +576,15 @@ args_dict = {
         "--original-xml %(CLONE_DISK_XML)s --auto-clone",
         # Auto flag w/ managed storage,
         "--original-xml %(CLONE_STORAGE_XML)s --auto-clone",
+        # Auto flag, actual VM, skip state check
+        "-o test-for-clone --auto-clone --clone-running",
       ],
 
       "invalid" : [
         # Just the auto flag
         "--auto-clone"
+        # Auto flag, actual VM, without state skip
+        "-o test-for-clone --auto-clone",
       ]
     }, # category "misc"
 
