@@ -50,7 +50,7 @@ URLTEST_ACCESS="TestURLFetch.*Access to a protected member"
 TEST_HACKS="TestClone.*protected member _util|testQEMUDriverName.*protected member _get_uri|Access to a protected member _util"
 
 # Scattered examples of legitimately unused arguments
-UNUSED_ARGS="(SuseDistro|SolarisDistro|NetWareDistro).isValidStore.*Unused argument 'progresscb'|.*Installer.prepare.*Unused argument|post_install_check.*Unused argument 'guest'|Guest.__init__.*Unused argument 'type'"
+UNUSED_ARGS="(SuseDistro|SolarisDistro|NetWareDistro).isValidStore.*Unused argument 'progresscb'|.*Installer.prepare.*Unused argument|post_install_check.*Unused argument 'guest'|Guest.__init__.*Unused argument 'type'|_get_bootdev"
 
 # Outside __init__ checks throw false positives with distutils custom commands
 # tests.storage also invokes false positives using hasattr
@@ -59,9 +59,6 @@ OUTSIDE_INIT="(.*Test.*|.*createPool.*)outside __init__"
 # pylint complains about some of the subclass funkiness in chardev classes
 CHAR_SUBCLASS=".*VirtualCharDevice' has no '(source_mode|source_path)' member.*|.*Method '_char_xml' is abstract in class 'VirtualCharDevice'.*"
 
-# Crap spewed by python 2.6
-MAX_RECURSION="maximum recursion depth"
-
 # FIXME: Everything skipped below are all bugs
 
 # Libvirt connect() method is broken for getting an objects connection, this
@@ -70,7 +67,7 @@ ACCESS__CONN="Access to a protected member _conn"
 
 # There isn't a clean API way to access this functions from the API, but
 # they provide info that is needed. These need need to be fixed.
-PROT_MEM_BUGS="protected member (_lookup_osdict_key|_OS_TYPES|_prepare_install|_create_devices|_add_install_dev|_cleanup_install)|'virtinst.FullVirtGuest' has no '_OS_TYPES'"
+PROT_MEM_BUGS="protected member (_lookup_osdict_key|_OS_TYPES|_prepare_install|_create_devices|_cleanup_install)|'virtinst.FullVirtGuest' has no '_OS_TYPES'|_install_bootconfig"
 
 DMSG=""
 addmsg() {
@@ -147,7 +144,6 @@ pylint --ignore=coverage.py, $FILES \
         -ve "$TEST_HACKS" \
         -ve "$PROT_MEM_BUGS" \
         -ve "$CHAR_SUBCLASS" \
-        -ve "$MAX_RECURSION" \
         -ve "$OUTSIDE_INIT" | \
   $AWK '\
 # Strip out any "*** Module name" lines if we dont list any errors for them
