@@ -89,26 +89,6 @@ def distroClass(distname):
 urls = {
 
     # Fedora Distros
-    "fedora7" : {
-        'i386'  : FEDORA_BASEURL % ("7", "i386"),
-        'x86_64': FEDORA_BASEURL % ("7", "x86_64"),
-        'distro': ("linux", "fedora7")
-    },
-    "fedora8" : {
-        'i386'  : FEDORA_BASEURL % ("8", "i386"),
-        'x86_64': FEDORA_BASEURL % ("8", "x86_64"),
-        'distro': ("linux", "fedora8")
-    },
-    "fedora9" : {
-        'i386'  : FEDORA_BASEURL % ("9", "i386"),
-        'x86_64': FEDORA_BASEURL % ("9", "x86_64"),
-        'distro': ("linux", "fedora9")
-    },
-    "fedora10" : {
-        'i386'  : FEDORA_BASEURL % ("10", "i386"),
-        'x86_64': FEDORA_BASEURL % ("10", "x86_64"),
-        'distro': ("linux", "fedora10")
-    },
     "fedora11" : {
         'i386'  : FEDORA_BASEURL % ("11", "i386"),
         'x86_64': FEDORA_BASEURL % ("11", "x86_64"),
@@ -119,6 +99,11 @@ urls = {
         'x86_64': FEDORA_BASEURL % ("12", "x86_64"),
         'distro': ("linux", "fedora12")
     },
+    "fedora13" : {
+        'i386'  : FEDORA_BASEURL % ("13", "i386"),
+        'x86_64': FEDORA_BASEURL % ("13", "x86_64"),
+        'distro': ("linux", "fedora13")
+    },
     "fedora-rawhide" : {
         #'i386'  : FEDORA_RAWHIDE_BASEURL % ("i386"),
         #'x86_64': FEDORA_RAWHIDE_BASEURL % ("x86_64"),
@@ -361,7 +346,7 @@ class TestURLFetch(unittest.TestCase):
                 logging.debug("Known lack of boot.iso in %s tree. Skipping." \
                               % distname)
             else:
-                boot = hvmstore.acquireBootDisk(fetcher, self.meter)
+                boot = hvmstore.acquireBootDisk(testguest, fetcher, self.meter)
                 logging.debug("acquireBootDisk: %s" % str(boot))
 
                 if boot != True:
