@@ -28,12 +28,14 @@ from virtinst import VirtualCharDevice
 from virtinst import VirtualVideoDevice
 from virtinst import VirtualController
 from virtinst import VirtualWatchdog
+from virtinst import VirtualInputDevice
 import tests
 
 conn = tests.open_testdriver()
+scratch = os.path.join(os.getcwd(), "tests", "testscratchdir")
 
-def get_basic_paravirt_guest():
-    g = virtinst.ParaVirtGuest(connection=conn, type="xen")
+def get_basic_paravirt_guest(testconn=conn, installer=None):
+    g = virtinst.ParaVirtGuest(connection=testconn, type="xen")
     g.name = "TestGuest"
     g.memory = int(200)
     g.maxmemory = int(400)
@@ -41,10 +43,15 @@ def get_basic_paravirt_guest():
     g.boot = ["/boot/vmlinuz","/boot/initrd"]
     g.graphics = (True, "vnc", None, "ja")
     g.vcpus = 5
+
+    if installer:
+        g.installer = installer
+
+    g.installer._scratchdir = scratch
     return g
 
-def get_basic_fullyvirt_guest(typ="xen"):
-    g = virtinst.FullVirtGuest(connection=conn, type=typ,
+def get_basic_fullyvirt_guest(typ="xen", testconn=conn, installer=None):
+    g = virtinst.FullVirtGuest(connection=testconn, type=typ,
                                emulator="/usr/lib/xen/bin/qemu-dm",
                                arch="i686")
     g.name = "TestGuest"
@@ -52,17 +59,53 @@ def get_basic_fullyvirt_guest(typ="xen"):
     g.maxmemory = int(400)
     g.uuid = "12345678-1234-1234-1234-123456789012"
     g.cdrom = "/dev/loop0"
-    g.set_os_type("other")
-    g.set_os_variant("generic")
     g.graphics = (True, "sdl")
     g.features['pae'] = 0
     g.vcpus = 5
+    if installer:
+        g.installer = installer
+
+    g.installer._scratchdir = scratch
     return g
 
-def get_floppy(path="/default-pool/testvol1.img"):
+def make_import_installer():
+    inst = virtinst.ImportInstaller(type="xen", os_type="hvm", conn=conn)
+    return inst
+
+def make_distro_installer(location="/default-pool/default-vol", gtype="xen"):
+    inst = virtinst.DistroInstaller(type=gtype, os_type="hvm", conn=conn,
+                                    location=location)
+    return inst
+
+def make_live_installer(location="/dev/loop0", gtype="xen"):
+    inst = virtinst.LiveCDInstaller(type=gtype, os_type="hvm",
+                                    conn=conn, location=location)
+    return inst
+
+def make_pxe_installer(gtype="xen"):
+    inst = virtinst.PXEInstaller(type=gtype, os_type="hvm", conn=conn)
+    return inst
+
+def build_win_kvm(path=None):
+    g = get_basic_fullyvirt_guest("kvm")
+    g.os_type = "windows"
+    g.os_variant = "winxp"
+    g.disks.append(get_filedisk(path))
+    g.disks.append(get_blkdisk())
+    g.nics.append(get_virtual_network())
+    g.add_device(VirtualAudio())
+    g.add_device(VirtualVideoDevice(g.conn))
+
+    return g
+
+def get_floppy(path = None):
+    if not path:
+        path = "/default-pool/testvol1.img"
     return VirtualDisk(path, conn=conn, device=VirtualDisk.DEVICE_FLOPPY)
 
-def get_filedisk(path="/tmp/test.img"):
+def get_filedisk(path = None):
+    if not path:
+        path = "/tmp/test.img"
     return VirtualDisk(path, size=.0001, conn=conn)
 
 def get_blkdisk():
@@ -81,35 +124,119 @@ def qemu_uri():
 def xen_uri():
     return "xen:///"
 
+def build_xmlfile(filebase):
+    if not filebase:
+        return None
+    return os.path.join("tests/xmlconfig-xml", filebase + ".xml")
+
+def sanitize_xml(xml):
+    # Libvirt throws errors since we are defining domain
+    # type='xen', when test driver can only handle type='test'
+    # Sanitize the XML so we can define
+    if not xml:
+        return xml
+
+    xml = xml.replace("<domain type='xen'>",
+                      "<domain type='test'>")
+    xml = xml.replace(">linux<", ">xen<")
+
+    return xml
+
 class TestXMLConfig(unittest.TestCase):
 
-    def _compare(self, xenguest, filebase, do_install, do_disk_boot=False):
-        filename = os.path.join("tests/xmlconfig-xml", filebase + ".xml")
-        xenguest._prepare_install(progress.BaseMeter())
-        try:
-            actualXML = xenguest.get_config_xml(install=do_install,
-                                                disk_boot=do_disk_boot)
-            tests.diff_compare(actualXML, filename)
-            # Libvirt throws errors since we are defining domain
-            # type='xen', when test driver can only handle type='test'
-            # Sanitize the XML so we can define
-            actualXML = actualXML.replace("<domain type='xen'>",
-                                          "<domain type='test'>")
-            actualXML = actualXML.replace(">linux<", ">xen<")
+    def tearDown(self):
+        if os.path.exists(scratch):
+            os.rmdir(scratch)
 
-            # Should probably break this out into a separate function
-            dom = xenguest.conn.defineXML(actualXML)
+    def _compare(self, guest, filebase, do_install, do_disk_boot=False):
+        filename = build_xmlfile(filebase)
+
+        guest._prepare_install(progress.BaseMeter())
+        try:
+            actualXML = guest.get_config_xml(install=do_install,
+                                             disk_boot=do_disk_boot)
+
+            tests.diff_compare(actualXML, filename)
+            self._testCreate(guest.conn, actualXML)
+        finally:
+            guest._cleanup_install()
+
+    def _testCreate(self, testconn, xml):
+        xml = sanitize_xml(xml)
+
+        dom = testconn.defineXML(xml)
+        try:
             dom.create()
             dom.destroy()
             dom.undefine()
-        finally:
-            xenguest._cleanup_install()
+        except:
+            try:
+                dom.destroy()
+            except:
+                pass
+            try:
+                dom.undefine()
+            except:
+                pass
 
-    def conn_function_wrappers(self, guest, filename, do_boot,
+    def _testInstall(self, guest,
+                     instxml=None, bootxml=None, contxml=None):
+        instname = build_xmlfile(instxml)
+        bootname = build_xmlfile(bootxml)
+        contname = build_xmlfile(contxml)
+        consolecb = None
+        meter = None
+        removeOld = None
+        wait = True
+        dom = None
+
+        old_getxml = guest.get_config_xml
+        def new_getxml(install=True, disk_boot=False):
+            xml = old_getxml(install, disk_boot)
+            return sanitize_xml(xml)
+        guest.get_config_xml = new_getxml
+
+        try:
+            dom = guest.start_install(consolecb, meter, removeOld, wait)
+            dom.destroy()
+
+            # Replace kernel/initrd with known info
+            if (guest.installer._install_bootconfig and
+                guest.installer._install_bootconfig.kernel):
+                guest.installer._install_bootconfig.kernel = "kernel"
+                guest.installer._install_bootconfig.initrd = "initrd"
+
+            xmlinst = guest.get_config_xml(True, False)
+            xmlboot = guest.get_config_xml(False, False)
+            xmlcont = guest.get_config_xml(True, True)
+
+            if instname:
+                tests.diff_compare(xmlinst, instname)
+            if contname:
+                tests.diff_compare(xmlcont, contname)
+            if bootname:
+                tests.diff_compare(xmlboot, bootname)
+
+            if guest.get_continue_inst():
+                guest.continue_install(consolecb, meter, wait)
+
+        finally:
+            if dom:
+                try:
+                    dom.destroy()
+                except:
+                    pass
+                try:
+                    dom.undefine()
+                except:
+                    pass
+
+
+    def conn_function_wrappers(self, guest, funcargs,
+                               func=None,
                                conn_version=None,
                                conn_uri=None,
-                               libvirt_version=None,
-                               do_disk_boot=False):
+                               libvirt_version=None):
         testconn = guest.conn
 
         def set_func(newfunc, funcname, obj, force=False):
@@ -137,7 +264,10 @@ class TestXMLConfig(unittest.TestCase):
             old_version = set_version(conn_version)
             old_uri = set_uri(conn_uri)
             old_libvirt_version = set_libvirt_version(libvirt_version)
-            self._compare(guest, filename, do_boot, do_disk_boot)
+
+            if not func:
+                func = self._compare
+            func(*funcargs)
         finally:
             set_version(*old_version)
             set_uri(*old_uri)
@@ -215,7 +345,6 @@ class TestXMLConfig(unittest.TestCase):
 
 
 
-
     def testInstallParavirtDiskFile(self):
         g = get_basic_paravirt_guest()
         g.disks.append(get_filedisk())
@@ -277,78 +406,147 @@ class TestXMLConfig(unittest.TestCase):
         self._compare(g, "install-fullyvirt-disk-block", True)
 
     def testInstallFVPXE(self):
-        g = get_basic_fullyvirt_guest()
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
         g.disks.append(get_filedisk())
         self._compare(g, "install-fullyvirt-pxe", True)
 
     def testBootFVPXE(self):
-        g = get_basic_fullyvirt_guest()
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
         g.disks.append(get_filedisk())
         self._compare(g, "boot-fullyvirt-pxe", False)
 
+    def testBootFVPXEAlways(self):
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+        g.disks.append(get_filedisk())
+
+        g.installer.bootconfig.bootorder = [
+            g.installer.bootconfig.BOOT_DEVICE_NETWORK]
+        g.installer.bootconfig.enable_bootmenu = True
+
+        self._compare(g, "boot-fullyvirt-pxe-always", False)
+
     def testInstallFVPXENoDisks(self):
-        g = get_basic_fullyvirt_guest()
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
         self._compare(g, "install-fullyvirt-pxe-nodisks", True)
 
     def testBootFVPXENoDisks(self):
-        g = get_basic_fullyvirt_guest()
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
         self._compare(g, "boot-fullyvirt-pxe-nodisks", False)
 
     def testInstallFVLiveCD(self):
-        g = get_basic_fullyvirt_guest()
-        g.installer = virtinst.LiveCDInstaller(type="xen", os_type="hvm",
-                                               conn=g.conn,
-                                               location="/dev/loop0")
+        i = make_live_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
         self._compare(g, "install-fullyvirt-livecd", False)
 
     def testDoubleInstall(self):
         # Make sure that installing twice generates the same XML, to ensure
         # we aren't polluting the device list during the install process
-        g = get_basic_fullyvirt_guest()
-        g.installer = virtinst.LiveCDInstaller(type="xen", os_type="hvm",
-                                               conn=g.conn,
-                                               location="/dev/loop0")
+        i = make_live_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
         self._compare(g, "install-fullyvirt-livecd", False)
         self._compare(g, "install-fullyvirt-livecd", False)
 
+    def testDefaultDeviceRemoval(self):
+        g = get_basic_fullyvirt_guest()
+        g.disks.append(get_filedisk())
+
+        inp = VirtualInputDevice(g.conn)
+        cons = VirtualCharDevice.get_dev_instance(conn,
+                                VirtualCharDevice.DEV_CONSOLE,
+                                VirtualCharDevice.CHAR_PTY)
+        g.add_device(inp)
+        g.add_device(cons)
+
+        g.remove_device(inp)
+        g.remove_device(cons)
+
+        self._compare(g, "boot-default-device-removal", False)
+
+    def testOSDeviceDefaultChange(self):
+        """
+        Make sure device defaults are properly changed if we change OS
+        distro/variant mid process
+        """
+        i = make_distro_installer(gtype="kvm")
+        g = get_basic_fullyvirt_guest("kvm", installer=i)
+
+        do_install = False
+        g.installer.cdrom = True
+        g.disks.append(get_floppy())
+        g.disks.append(get_filedisk())
+        g.disks.append(get_blkdisk())
+        g.nics.append(get_virtual_network())
+
+        # Call get_config_xml to set first round of defaults without an
+        # os_variant set
+        fargs = (do_install,)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri,
+                                    func=g.get_config_xml)
+
+        g.os_variant = "fedora11"
+        fargs = (g, "install-f11", do_install)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
     def testInstallFVImport(self):
-        g = get_basic_fullyvirt_guest()
+        i = make_import_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+
         g.disks.append(get_filedisk())
-        g.installer = virtinst.ImportInstaller(type="xen", os_type="hvm",
-                                               conn=g.conn)
         self._compare(g, "install-fullyvirt-import", False)
 
+    def testInstallFVImportKernel(self):
+        i = make_import_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+
+        g.disks.append(get_filedisk())
+        g.installer.bootconfig.kernel = "kernel"
+        g.installer.bootconfig.initrd = "initrd"
+        g.installer.bootconfig.kernel_args = "my kernel args"
+
+        self._compare(g, "install-fullyvirt-import-kernel", False)
+
+    def testInstallFVImportMulti(self):
+        i = make_import_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+
+        g.installer.bootconfig.enable_bootmenu = False
+        g.installer.bootconfig.bootorder = ["hd", "fd", "cdrom", "network"]
+        g.disks.append(get_filedisk())
+        self._compare(g, "install-fullyvirt-import-multiboot", False)
+
     def testInstallPVImport(self):
-        g = get_basic_paravirt_guest()
+        i = make_import_installer()
+        g = get_basic_paravirt_guest(installer=i)
+
         g.disks.append(get_filedisk())
-        g.installer = virtinst.ImportInstaller(type="xen", os_type="xen",
-                                               conn=g.conn)
         self._compare(g, "install-paravirt-import", False)
 
     def testQEMUDriverName(self):
         g = get_basic_fullyvirt_guest()
         g.disks.append(get_blkdisk())
-        self.conn_function_wrappers(g, "misc-qemu-driver-name", True,
-                                    conn_uri=qemu_uri)
+        fargs = (g, "misc-qemu-driver-name", True)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
         g = get_basic_fullyvirt_guest()
         g.disks.append(get_filedisk())
-        self.conn_function_wrappers(g, "misc-qemu-driver-type", True,
-                                    conn_uri=qemu_uri)
+        fargs = (g, "misc-qemu-driver-type", True)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
         g = get_basic_fullyvirt_guest()
         g.disks.append(get_filedisk("/default-pool/iso-vol"))
-        self.conn_function_wrappers(g, "misc-qemu-iso-disk", True,
-                                    conn_uri=qemu_uri)
+        fargs = (g, "misc-qemu-iso-disk", True)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
+
+        g = get_basic_fullyvirt_guest()
+        g.disks.append(get_filedisk("/default-pool/iso-vol"))
+        g.disks[0].driver_type = "qcow2"
+        fargs = (g, "misc-qemu-driver-overwrite", True)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
     def testXMLEscaping(self):
         g = get_basic_fullyvirt_guest()
@@ -357,39 +555,38 @@ class TestXMLConfig(unittest.TestCase):
 
     # OS Type/Version configurations
     def testF10(self):
-        g = get_basic_fullyvirt_guest("kvm")
+        i = make_pxe_installer(gtype="kvm")
+        g = get_basic_fullyvirt_guest("kvm", installer=i)
+
         g.os_type = "linux"
         g.os_variant = "fedora10"
-        g.installer = virtinst.PXEInstaller(type="kvm", os_type="hvm",
-                                            conn=g.conn)
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.nics.append(get_virtual_network())
-        self.conn_function_wrappers(g, "install-f10", True,
-                                    conn_uri=qemu_uri)
+        fargs = (g, "install-f10", True)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
     def testF11(self):
-        g = get_basic_fullyvirt_guest("kvm")
+        i = make_distro_installer(gtype="kvm")
+        g = get_basic_fullyvirt_guest("kvm", installer=i)
+
         g.os_type = "linux"
         g.os_variant = "fedora11"
-        g.installer = virtinst.DistroInstaller(type="kvm", os_type="hvm",
-                                               conn=g.conn,
-                                               location="/default-pool/default-vol")
         g.installer.cdrom = True
         g.disks.append(get_floppy())
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.nics.append(get_virtual_network())
-        self.conn_function_wrappers(g, "install-f11", False,
-                                    conn_uri=qemu_uri)
+        fargs = (g, "install-f11", False)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
     def testF11AC97(self):
         def build_guest():
-            g = get_basic_fullyvirt_guest("kvm")
+            i = make_distro_installer(gtype="kvm")
+            g = get_basic_fullyvirt_guest("kvm", installer=i)
+
             g.os_type = "linux"
             g.os_variant = "fedora11"
-            g.installer = virtinst.DistroInstaller(type="kvm", os_type="hvm",
-                            conn=g.conn, location="/default-pool/default-vol")
             g.installer.cdrom = True
             g.disks.append(get_floppy())
             g.disks.append(get_filedisk())
@@ -411,78 +608,66 @@ class TestXMLConfig(unittest.TestCase):
             return 11000
 
         g = build_guest()
-        self.conn_function_wrappers(g, "install-f11-ac97", False,
+        fargs = (g, "install-f11-ac97", False)
+        self.conn_function_wrappers(g, fargs,
                                     conn_uri=qemu_uri,
                                     conn_version=conn_support_ac97)
 
         g = build_guest()
-        self.conn_function_wrappers(g, "install-f11-noac97", False,
+        fargs = (g, "install-f11-noac97", False)
+        self.conn_function_wrappers(g, fargs,
                                     libvirt_version=libvirt_nosupport_ac97,
                                     conn_uri=qemu_uri)
 
         g = build_guest()
-        self.conn_function_wrappers(g, "install-f11-noac97", False,
+        fargs = (g, "install-f11-noac97", False)
+        self.conn_function_wrappers(g, fargs,
                                     conn_version=conn_nosupport_ac97,
                                     conn_uri=qemu_uri)
 
 
     def testF11Qemu(self):
-        g = get_basic_fullyvirt_guest("qemu")
+        i = make_distro_installer(gtype="qemu")
+        g = get_basic_fullyvirt_guest("qemu", installer=i)
+
         g.os_type = "linux"
         g.os_variant = "fedora11"
-        g.installer = virtinst.DistroInstaller(type="qemu", os_type="hvm",
-                                               conn=g.conn,
-                                               location="/default-pool/default-vol")
         g.installer.cdrom = True
         g.disks.append(get_floppy())
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.nics.append(get_virtual_network())
-        self.conn_function_wrappers(g, "install-f11-qemu", False,
-                                    conn_uri=qemu_uri)
+        fargs = (g, "install-f11-qemu", False)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
     def testF11Xen(self):
-        g = get_basic_fullyvirt_guest("xen")
+        i = make_distro_installer(gtype="xen")
+        g = get_basic_fullyvirt_guest("xen", installer=i)
+
         g.os_type = "linux"
         g.os_variant = "fedora11"
-        g.installer = virtinst.DistroInstaller(type="xen", os_type="hvm",
-                                               conn=g.conn,
-                                               location="/default-pool/default-vol")
         g.installer.cdrom = True
         g.disks.append(get_floppy())
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.nics.append(get_virtual_network())
-        self.conn_function_wrappers(g, "install-f11-xen", False,
-                                    conn_uri=xen_uri)
-
-    def _build_win_kvm(self):
-        g = get_basic_fullyvirt_guest("kvm")
-        g.os_type = "windows"
-        g.os_variant = "winxp"
-        g.disks.append(get_filedisk())
-        g.disks.append(get_blkdisk())
-        g.nics.append(get_virtual_network())
-        g.add_device(VirtualAudio())
-        g.add_device(VirtualVideoDevice(g.conn))
-
-        return g
+        fargs = (g, "install-f11-xen", False)
+        self.conn_function_wrappers(g, fargs, conn_uri=xen_uri)
 
     def testInstallWindowsKVM(self):
-        g = self._build_win_kvm()
-        self.conn_function_wrappers(g, "winxp-kvm-stage1", True,
-                                    conn_uri=qemu_uri)
+        g = build_win_kvm("/default-pool/winxp.img")
+        fargs = (g, "winxp-kvm-stage1", True)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
     def testContinueWindowsKVM(self):
-        g = self._build_win_kvm()
-        self.conn_function_wrappers(g, "winxp-kvm-stage2", True,
-                                    conn_uri=qemu_uri,
-                                    do_disk_boot=True)
+        g = build_win_kvm("/default-pool/winxp.img")
+        fargs = (g, "winxp-kvm-stage2", True, True)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
     def testBootWindowsKVM(self):
-        g = self._build_win_kvm()
-        self.conn_function_wrappers(g, "winxp-kvm-stage3", False,
-                                    conn_uri=qemu_uri)
+        g = build_win_kvm("/default-pool/winxp.img")
+        fargs = (g, "winxp-kvm-stage3", False)
+        self.conn_function_wrappers(g, fargs, conn_uri=qemu_uri)
 
 
     def testInstallWindowsXenNew(self):
@@ -504,34 +689,42 @@ class TestXMLConfig(unittest.TestCase):
         for f, xml in [(old_xen_ver, "install-windowsxp-xenold"),
                        (new_xen_ver, "install-windowsxp-xennew")]:
 
-            self.conn_function_wrappers(g, xml, True,
-                                        conn_version=f,
-                                        conn_uri=xen_uri)
+            fargs = (g, xml, True)
+            self.conn_function_wrappers(g, fargs,
+                                        conn_version=f, conn_uri=xen_uri)
 
 
     # Device heavy configurations
     def testManyDisks2(self):
-        g = get_basic_fullyvirt_guest()
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+
         g.disks.append(get_filedisk())
         g.disks.append(get_blkdisk())
         g.disks.append(VirtualDisk(conn=g.conn, path="/dev/loop0",
-                                   device=VirtualDisk.DEVICE_CDROM))
+                                   device=VirtualDisk.DEVICE_CDROM,
+                                   driverType="raw"))
+        g.disks.append(VirtualDisk(conn=g.conn, path="/dev/loop0",
+                                   device=VirtualDisk.DEVICE_DISK,
+                                   driverName="qemu", format="qcow2"))
         g.disks.append(VirtualDisk(conn=g.conn, path=None,
                                    device=VirtualDisk.DEVICE_CDROM,
                                    bus="scsi"))
         g.disks.append(VirtualDisk(conn=g.conn, path=None,
                                    device=VirtualDisk.DEVICE_FLOPPY))
         g.disks.append(VirtualDisk(conn=g.conn, path="/dev/loop0",
-                                   device=VirtualDisk.DEVICE_FLOPPY))
+                                   device=VirtualDisk.DEVICE_FLOPPY,
+                                   driverName="phy", driverCache="none"))
         g.disks.append(VirtualDisk(conn=g.conn, path="/dev/loop0",
-                                   bus="virtio"))
+                                   bus="virtio", driverName="qemu",
+                                   driverType="qcow2", driverCache="none"))
 
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
         self._compare(g, "boot-many-disks2", False)
 
     def testManyNICs(self):
-        g = get_basic_fullyvirt_guest()
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+
         net1 = VirtualNetworkInterface(type="user",
                                        macaddr="11:11:11:11:11:11")
         net2 = get_virtual_network()
@@ -544,12 +737,12 @@ class TestXMLConfig(unittest.TestCase):
         g.nics.append(net2)
         g.nics.append(net3)
         g.nics.append(net4)
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
         self._compare(g, "boot-many-nics", False)
 
     def testManyHostdevs(self):
-        g = get_basic_fullyvirt_guest()
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+
         dev1 = VirtualHostDeviceUSB(g.conn)
         dev1.product = "0x1234"
         dev1.vendor = "0x4321"
@@ -561,23 +754,23 @@ class TestXMLConfig(unittest.TestCase):
 
         g.hostdevs.append(dev1)
         g.hostdevs.append(dev2)
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
         self._compare(g, "boot-many-hostdevs", False)
 
     def testManySounds(self):
-        g = get_basic_fullyvirt_guest()
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+
         g.sound_devs.append(VirtualAudio("sb16", conn=g.conn))
         g.sound_devs.append(VirtualAudio("es1370", conn=g.conn))
         g.sound_devs.append(VirtualAudio("pcspk", conn=g.conn))
         g.sound_devs.append(VirtualAudio(conn=g.conn))
 
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
         self._compare(g, "boot-many-sounds", False)
 
     def testManyChars(self):
-        g = get_basic_fullyvirt_guest()
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+
         dev1 = VirtualCharDevice.get_dev_instance(g.conn,
                                                   VirtualCharDevice.DEV_SERIAL,
                                                   VirtualCharDevice.CHAR_NULL)
@@ -599,16 +792,29 @@ class TestXMLConfig(unittest.TestCase):
         dev4.source_host = "my.source.host"
         dev4.source_port = "2222"
 
+        dev5 = VirtualCharDevice.get_dev_instance(g.conn,
+                                                  VirtualCharDevice.DEV_CHANNEL,
+                                                  VirtualCharDevice.CHAR_PTY)
+        dev5.target_type = dev5.CHAR_CHANNEL_TARGET_VIRTIO
+        dev5.target_name = "foo.bar.frob"
+
+        dev6 = VirtualCharDevice.get_dev_instance(g.conn,
+                                                  VirtualCharDevice.DEV_CONSOLE,
+                                                  VirtualCharDevice.CHAR_PTY)
+        dev6.target_type = dev5.CHAR_CONSOLE_TARGET_VIRTIO
+
         g.add_device(dev1)
         g.add_device(dev2)
         g.add_device(dev3)
         g.add_device(dev4)
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
+        g.add_device(dev5)
+        g.add_device(dev6)
         self._compare(g, "boot-many-chars", False)
 
     def testManyDevices(self):
-        g = get_basic_fullyvirt_guest()
+        i = make_pxe_installer()
+        g = get_basic_fullyvirt_guest(installer=i)
+
         g.description = "foooo barrrr \n baz && snarf. '' \"\" @@$\n"
 
         # Hostdevs
@@ -687,9 +893,44 @@ class TestXMLConfig(unittest.TestCase):
         seclabel.imagelabel = "imagelabel"
         g.seclabel = seclabel
 
-        g.installer = virtinst.PXEInstaller(type="xen", os_type="hvm",
-                                            conn=g.conn)
         self._compare(g, "boot-many-devices", False)
 
+    def testCpuset(self):
+        testconn = libvirt.open("test:///default")
+        g = get_basic_fullyvirt_guest(testconn=testconn)
+
+        # Cpuset
+        cpustr = g.generate_cpuset(g.conn, g.memory)
+        g.cpuset = cpustr
+
+        self._compare(g, "boot-cpuset", False)
+
+
+    #
+    # Full Install tests: try to mimic virt-install as much as possible
+    #
+
+    def testFullKVMRHEL6(self):
+        i = make_distro_installer(location="tests/cli-test-xml/fakerhel6tree",
+                                  gtype="kvm")
+        g = get_basic_fullyvirt_guest("kvm", installer=i)
+        g.disks.append(get_floppy())
+        g.disks.append(get_filedisk("/default-pool/rhel6.img"))
+        g.disks.append(get_blkdisk())
+        g.nics.append(get_virtual_network())
+        g.add_device(VirtualAudio())
+        g.add_device(VirtualVideoDevice(g.conn))
+        g.os_autodetect = True
+
+        fargs = (g, "rhel6-kvm-stage1", "rhel6-kvm-stage2")
+        self.conn_function_wrappers(g, fargs, func=self._testInstall,
+                                    conn_uri=qemu_uri)
+
+    def testFullKVMWinxp(self):
+        g = build_win_kvm("/default-pool/winxp.img")
+        fargs = (g, "winxp-kvm-stage1", "winxp-kvm-stage3", "winxp-kvm-stage2")
+        self.conn_function_wrappers(g, fargs, func=self._testInstall,
+                                    conn_uri=qemu_uri)
+
 if __name__ == "__main__":
     unittest.main()
