@@ -22,21 +22,25 @@
 import os, os.path
 import time
 import re
+import logging
+import signal
+import copy
+
 import urlgrabber.progress as progress
-import _util
 import libvirt
+
+import _util
 import CapabilitiesParser
 import VirtualGraphics
 import support
 from VirtualDevice import VirtualDevice
 from VirtualDisk import VirtualDisk
+from VirtualInputDevice import VirtualInputDevice
 from Clock import Clock
 from Seclabel import Seclabel
 
 import osdict
 from virtinst import _virtinst as _
-import logging
-import signal
 
 def _validate_cpuset(conn, val):
     if val is None or val == "":
@@ -109,6 +113,60 @@ class Guest(object):
         return tuple(pinlist)
     cpuset_str_to_tuple = staticmethod(cpuset_str_to_tuple)
 
+    @staticmethod
+    def generate_cpuset(conn, mem):
+        """
+        Generates a cpu pinning string based on host NUMA configuration.
+
+        If host doesn't have a suitable NUMA configuration, a RuntimeError
+        is thrown.
+        """
+        caps = CapabilitiesParser.parse(conn.getCapabilities())
+
+        if caps.host.topology is None:
+            raise RuntimeError(_("No topology section in capabilities xml."))
+
+        cells = caps.host.topology.cells
+        if len(cells) <= 1:
+            raise RuntimeError(_("Capabilities only show <= 1 cell. "
+                                 "Not NUMA capable"))
+
+        # Capabilities tells us about the available memory 'cells' on the
+        # system. Each 'cell' has associated 'cpu's.
+        #
+        # Use getCellsFreeMemory to determine which 'cell' has the smallest
+        # amount of memory which fits the requested VM memory amount, then
+        # pin the VM to that 'cell's associated 'cpu's
+
+        cell_mem = conn.getCellsFreeMemory(0, len(cells))
+        cell_id = -1
+        mem = mem * 1024
+        for i in range(len(cells)):
+            if cell_mem[i] < mem:
+                # Cell doesn't have enough mem to fit, skip it
+                continue
+
+            if len(cells[i].cpus) == 0:
+                # No cpus to use for the cell
+                continue
+
+            # Find smallest cell that fits
+            if cell_id < 0 or cell_mem[i] < cell_mem[cell_id]:
+                cell_id = i
+
+        if cell_id < 0:
+            raise RuntimeError(_("Could not find any usable NUMA "
+                                 "cell/cpu combinations."))
+
+        # Build cpuset string
+        cpustr = ""
+        for cpu in cells[cell_id].cpus:
+            if cpustr != "":
+                cpustr += ","
+            cpustr += str(cpu.id)
+
+        return cpustr
+
     def __init__(self, type=None, connection=None, hypervisorURI=None,
                  installer=None):
 
@@ -169,13 +227,19 @@ class Guest(object):
         # Default bus for disks (set in subclass)
         self._diskbus = None
 
-        # Indicates that default devices have been assigned, so look for
-        # the user to overwrite
-        self._default_console_assigned = None
-        self._default_input_assigned = None
-
         self._caps = CapabilitiesParser.parse(self.conn.getCapabilities())
 
+        # Add default devices (if applicable)
+        self._default_input_device = None
+        self._default_console_device = None
+        inp = self._get_default_input_device()
+        con = self._get_default_console_device()
+        if inp:
+            self.add_device(inp)
+            self._default_input_device = inp
+        if con:
+            self.add_device(con)
+            self._default_console_device = con
 
     ######################
     # Property accessors #
@@ -306,8 +370,6 @@ class Guest(object):
             raise ValueError, _("OS type '%s' does not exist in our "
                                 "dictionary") % val
 
-        # Default may have changed with OS info
-        self._set_default_input_dev()
     os_type = property(get_os_type, set_os_type)
 
     def get_os_variant(self):
@@ -338,8 +400,6 @@ class Guest(object):
             if not found:
                 raise ValueError, _("Unknown OS variant '%s'" % val)
 
-        # Default may have changed with OS info
-        self._set_default_input_dev()
     os_variant = property(get_os_variant, set_os_variant)
 
     def set_os_autodetect(self, val):
@@ -517,21 +577,17 @@ class Guest(object):
             raise ValueError(_("Must pass a VirtualDevice instance."))
         devtype = dev.virtual_device_type
 
-        # Handling for back compat default device behavior
-        if ((devtype == VirtualDevice.VIRTUAL_DEV_CONSOLE or
-             devtype == VirtualDevice.VIRTUAL_DEV_SERIAL) and
-            self._default_console_assigned == True):
-
-            rmdev = self.get_devices(VirtualDevice.VIRTUAL_DEV_CONSOLE)[0]
-            self.remove_device(rmdev)
-            self._default_console_assigned = False
-
-        if (devtype == VirtualDevice.VIRTUAL_DEV_INPUT and
-            self._default_input_assigned == True):
-
-            rmdev = self.get_devices(VirtualDevice.VIRTUAL_DEV_INPUT)[0]
-            self.remove_device(rmdev)
-            self._default_input_assigned = False
+        # If user adds a device conflicting with a default assigned device
+        # remove the default
+        if (dev.virtual_device_type == VirtualDevice.VIRTUAL_DEV_INPUT and
+            self._default_input_device):
+            self.remove_device(self._default_input_device)
+            self._default_input_device = None
+        if (dev.virtual_device_type in [VirtualDevice.VIRTUAL_DEV_CONSOLE,
+                                        VirtualDevice.VIRTUAL_DEV_SERIAL] and
+            self._default_console_device):
+            self.remove_device(self._default_console_device)
+            self._default_console_device = None
 
         # Actually add the device
         if   devtype == VirtualDevice.VIRTUAL_DEV_DISK:
@@ -556,17 +612,20 @@ class Guest(object):
                         VirtualDevice.virtual_device_types)
         """
         if   devtype == VirtualDevice.VIRTUAL_DEV_DISK:
-            return self.disks[:]
+            devlist = self.disks[:]
         elif devtype == VirtualDevice.VIRTUAL_DEV_NET:
-            return self.nics[:]
+            devlist = self.nics[:]
         elif devtype == VirtualDevice.VIRTUAL_DEV_AUDIO:
-            return self.sound_devs[:]
+            devlist = self.sound_devs[:]
         elif devtype == VirtualDevice.VIRTUAL_DEV_GRAPHICS:
-            return self._graphics_dev and [self._graphics_dev] or []
+            devlist = self._graphics_dev and [self._graphics_dev] or []
         elif devtype == VirtualDevice.VIRTUAL_DEV_HOSTDEV:
-            return self.hostdevs[:]
+            devlist = self.hostdevs[:]
         else:
-            return self._dev_build_list(devtype)
+            devlist = self._dev_build_list(devtype)
+
+        devlist.extend(self._install_devices)
+        return self._dev_build_list(devtype, devlist)
 
     def get_all_devices(self):
         """
@@ -589,7 +648,7 @@ class Guest(object):
             found = True
 
         for devlist in [self.disks, self.nics, self.sound_devs, self.hostdevs,
-                        self._devices]:
+                        self._devices, self._install_devices]:
             if found:
                 break
 
@@ -601,49 +660,25 @@ class Guest(object):
         if not found:
             raise ValueError(_("Did not find device %s") % str(dev))
 
-        if (dev.virtual_device_type == VirtualDevice.VIRTUAL_DEV_INPUT and
-            self._default_input_assigned == True):
-            self._default_input_assigned = False
-        if (dev.virtual_device_type in [VirtualDevice.VIRTUAL_DEV_CONSOLE,
-                                        VirtualDevice.VIRTUAL_DEV_SERIAL] and
-            self._default_console_assigned == True):
-            self._default_console_assigned = False
-
-
-    # Device fetching functions used internally during the install process.
-    # These allow us to change dev defaults, add install media, etc. during
-    # the install, but revert to a clean state if the install fails
-    def _get_install_devs(self, devtype):
-        return self._dev_build_list(devtype, self._install_devices)
-
-    def _add_install_dev(self, dev):
-        self._install_devices.append(dev)
-
-    def _get_all_install_devs(self):
-        retlist = []
-        for devtype in VirtualDevice.virtual_device_types:
-            retlist.extend(self._get_install_devs(devtype))
-        return retlist
-
 
     ################################
     # Private xml building methods #
     ################################
 
-    def _get_input_device(self):
-        """ Return a tuple of the form (devtype, bus) for the desired
-            input device. (Must be implemented in subclass) """
-        raise NotImplementedError
+    def _get_default_input_device(self):
+        """
+        Return a VirtualInputDevice.
+        """
+        dev = VirtualInputDevice(self.conn)
+        return dev
 
-    def _get_device_xml(self, install=True):
-        # If install hasn't been prepared yet, make sure we use
-        # the regular device list
-        inst_devs = self._get_all_install_devs()
-        reg_devs = self.get_all_devices()
-        if len(inst_devs) >= len(reg_devs):
-            devs = inst_devs[:]
-        else:
-            devs = reg_devs[:]
+    def _get_default_console_device(self):
+        """
+        Only implemented for FullVirtGuest
+        """
+        return None
+
+    def _get_device_xml(self, devs, install=True):
 
         def do_remove_media(d):
             # Keep cdrom around, but with no media attached,
@@ -655,10 +690,20 @@ class Guest(object):
                     and not install and
                     not self.get_continue_inst())
 
+        def do_skip_disk(d):
+            # Skip transient labeled non-media disks
+            return (d.virtual_device_type == VirtualDevice.VIRTUAL_DEV_DISK and
+                    d.device == VirtualDisk.DEVICE_DISK
+                    and d.transient
+                    and not install)
+
         # Wrapper for building disk XML, handling transient CDROMs
         def get_dev_xml(dev):
             origpath = None
             try:
+                if do_skip_disk(dev):
+                    return ""
+
                 if do_remove_media(dev):
                     origpath = dev.path
                     dev.path = None
@@ -668,7 +713,7 @@ class Guest(object):
                 if origpath:
                     dev.path = origpath
 
-        xml = ""
+        xml = self._get_emulator_xml()
         # Build XML
         for dev in devs:
             xml = _util.xml_append(xml, get_dev_xml(dev))
@@ -692,6 +737,9 @@ class Guest(object):
                 features[f] = val
         return features
 
+    def _get_emulator_xml(self):
+        return ""
+
     def _get_features_xml(self):
         """
         Return features (pae, acpi, apic) xml
@@ -758,32 +806,24 @@ class Guest(object):
     ############################
 
     def _prepare_install(self, meter):
-        # Initialize install device list
-        self._install_devices = self.get_all_devices()[:]
-
-        # Set regular device defaults
-        self.set_defaults()
-
-        self._installer.prepare(guest = self,
-                                meter = meter)
-        if self._installer.install_disk is not None:
-            self._add_install_dev(self._installer.install_disk)
-
-        # Run 'set_defaults' after install prep, since some installers
-        # (ImageInstaller) alter the device list.
-        self._set_defaults(self._get_install_devs)
-
-    def _cleanup_install(self):
-        # Empty install dev list
         self._install_devices = []
 
+        # Fetch install media, prepare installer devices
+        self._installer.prepare(guest = self,
+                                meter = meter)
+
+        # Initialize install device list
+        for dev in self._installer.install_devices:
+            self._install_devices.append(dev)
+
+    def _cleanup_install(self):
         self._installer.cleanup()
 
     def _create_devices(self, progresscb):
         """
         Ensure that devices are setup
         """
-        for dev in self._get_all_install_devs():
+        for dev in self.get_all_devices():
             dev.setup_dev(self.conn, progresscb)
 
     ##############
@@ -806,9 +846,20 @@ class Guest(object):
                           this.)
         @type disk_boot: C{bool}
         """
+        # We do a shallow copy of the device list here, and set the defaults.
+        # This way, default changes aren't persistent, and we don't need
+        # to worry about when to call set_defaults
+        origdevs = self.get_all_devices()
+        devs = []
+        for dev in origdevs:
+            newdev = copy.copy(dev)
+            devs.append(newdev)
+
+        def get_transient_devices(devtype):
+            return self._dev_build_list(devtype, devs)
 
         # Set device defaults so we can validly generate XML
-        self._set_defaults(self.get_devices)
+        self._set_defaults(get_transient_devices)
 
         if install:
             action = "destroy"
@@ -852,7 +903,7 @@ class Guest(object):
         xml = add("  <on_crash>%s</on_crash>" % action)
         xml = add("  <vcpu%s>%d</vcpu>" % (cpuset, self.vcpus))
         xml = add("  <devices>")
-        xml = add(self._get_device_xml(install))
+        xml = add(self._get_device_xml(devs, install))
         xml = add("  </devices>")
         xml = add(self._get_seclabel_xml())
         xml = add("</domain>\n")
@@ -1129,31 +1180,28 @@ class Guest(object):
     # Device defaults #
     ###################
 
-    def _set_default_input_dev(self):
-        # This is called at init time, but also whenever the OS changes,
-        # since the input dev maybe be dependent on OS
-        if self._default_input_assigned == False:
-            return
-
-        for d in self.get_devices(VirtualDevice.VIRTUAL_DEV_INPUT):
-            self.remove_device(d)
-
-        # Add default input device
-        self._default_input_assigned = False
-        self.add_device(self._get_input_device())
-        self._default_input_assigned = True
-
     def set_defaults(self):
         """
         Public function to set guest defaults. Things like preferred
-        disk bus (unless one is specified). This will be called by the
-        install process, but can be called earlier if needed
+        disk bus (unless one is specified). These changes are persistent.
+        The install process will call a non-persistent version, so calling
+        this manually isn't required.
         """
         self._set_defaults(self.get_devices)
 
     def _set_defaults(self, devlist_func):
         soundtype = VirtualDevice.VIRTUAL_DEV_AUDIO
         videotype = VirtualDevice.VIRTUAL_DEV_VIDEO
+        inputtype = VirtualDevice.VIRTUAL_DEV_INPUT
+
+        # Set default input values
+        input_type = self._lookup_device_param(inputtype, "type")
+        input_bus = self._lookup_device_param(inputtype, "bus")
+        for inp in devlist_func(inputtype):
+            if (inp.type == inp.INPUT_TYPE_DEFAULT and
+                inp.bus  == inp.INPUT_BUS_DEFAULT):
+                inp.type = input_type
+                inp.bus  = input_bus
 
         # Generate disk targets, and set preferred disk bus
         used_targets = []
