@@ -63,7 +63,7 @@ def _qemu_sanitize_drvtype(phystype, fmt):
     """
     raw_list = [ "iso" ]
 
-    if phystype == VirtualDisk.TYPE_BLOCK:
+    if phystype == VirtualDisk.TYPE_BLOCK and not fmt:
         return VirtualDisk.DRIVER_QEMU_RAW
 
     if fmt in raw_list:
@@ -713,7 +713,11 @@ class VirtualDisk(VirtualDevice):
             if driver.lower() == "qemu":
                 drvname = self.DRIVER_QEMU
 
-        if self.vol_object:
+        if self.format:
+            if drvname == self.DRIVER_QEMU:
+                drvtype = _qemu_sanitize_drvtype(self.type, self.format)
+
+        elif self.vol_object:
             fmt = _util.get_xml_path(self.vol_object.XMLDesc(0),
                                      "/volume/target/format/@type")
             if drvname == self.DRIVER_QEMU:
@@ -734,11 +738,13 @@ class VirtualDisk(VirtualDevice):
                 drvname = self.DRIVER_TAP
                 drvtype = self.DRIVER_TAP_VDISK
 
+        # User already set driverName to a different value, respect that
         if self._driverName and self._driverName != drvname:
-            # User already set driverName to a different value, respect that
             return
-
         self._driverName = drvname or None
+
+        if self._driverType and self._driverType != drvtype:
+            return
         self._driverType = drvtype or None
 
     def __lookup_vol_name(self, name_tuple):
@@ -797,13 +803,15 @@ class VirtualDisk(VirtualDevice):
 
         def lookup_vol_by_path():
             try:
-                return self.conn.storageVolLookupByPath(self.path)
-            except:
-                return None
+                vol = self.conn.storageVolLookupByPath(self.path)
+                vol.info()
+                return vol, None
+            except Exception, e:
+                return None, e
 
         pool = _util.lookup_pool_by_path(self.conn,
                                          os.path.dirname(self.path))
-        vol = lookup_vol_by_path()
+        vol = lookup_vol_by_path()[0]
 
 
         # Is pool running?
@@ -811,24 +819,17 @@ class VirtualDisk(VirtualDevice):
             pool = None
 
         # Attempt to lookup path as a storage volume
-        try:
-            if vol:
-                vol.info()
-        except:
+        if pool and not vol:
             try:
-                try:
-                    # Pool may need to be refreshed, but if it errors,
-                    # invalidate it
-                    if pool:
-                        pool.refresh(0)
-                except:
-                    pool = None
-                    raise
+                # Pool may need to be refreshed, but if it errors,
+                # invalidate it
+                if pool:
+                    pool.refresh(0)
 
-                vol = self.conn.storageVolLookupByPath(self.path)
-                vol.info()
+                vol, verr = lookup_vol_by_path()
             except Exception, e:
                 vol = None
+                pool = None
                 verr = str(e)
 
         if vol:
@@ -1216,17 +1217,19 @@ class VirtualDisk(VirtualDevice):
         if not dname and self.driver_cache:
             self.driver_name = "qemu"
 
-        if path and not self.driver_name is None:
-            dtypexml = ""
+        if path:
+            drvxml = ""
+            if not self.driver_name is None:
+                drvxml += " name='%s'" % self.driver_name
+
             if not self.driver_type is None:
-                dtypexml = " type='%s'" % self.driver_type
+                drvxml += " type='%s'" % self.driver_type
 
-            dcachexml = ""
             if not self.driver_cache is None:
-                dcachexml = " cache='%s'" % self.driver_cache
+                drvxml += " cache='%s'" % self.driver_cache
 
-            ret += "      <driver name='%s'%s%s/>\n" % (self.driver_name,
-                                                      dtypexml, dcachexml)
+            if drvxml:
+                ret += "      <driver%s/>\n" % drvxml
 
         if path is not None:
             ret += "      <source %s='%s'/>\n" % (typeattr, path)
