@@ -29,9 +29,8 @@ from optparse import OptionValueError, OptionParser
 import libvirt
 import _util
 import virtinst
-from virtinst import CapabilitiesParser, VirtualNetworkInterface, \
-                     VirtualGraphics, VirtualAudio, VirtualDisk, User, \
-                     VirtualVideoDevice
+from virtinst import VirtualNetworkInterface, VirtualVideoDevice, Guest, \
+                     VirtualGraphics, VirtualAudio, VirtualDisk, User
 from virtinst import _virtinst as _
 
 MIN_RAM = 64
@@ -191,11 +190,12 @@ def setupLogging(appname, debug=False):
     # Log the app command string
     logging.debug("Launched with command line:\n%s" % " ".join(sys.argv))
 
-def fail(msg):
+def fail(msg, do_exit=True):
     """Convenience function when failing in cli app"""
     logging.error(msg)
     _util.log_exception()
-    _fail_exit()
+    if do_exit:
+        _fail_exit()
 
 def _fail_exit():
     sys.exit(1)
@@ -352,13 +352,19 @@ def prompt_for_input(noprompt_err, prompt = "", val = None, failed=False):
     print prompt + " ",
     return sys.stdin.readline().strip()
 
-def yes_or_no(s):
+def yes_or_no_convert(s):
     s = s.lower()
     if s in ("y", "yes", "1", "true", "t"):
         return True
     elif s in ("n", "no", "0", "false", "f"):
         return False
-    raise ValueError, "A yes or no response is required"
+    return None
+
+def yes_or_no(s):
+    ret = yes_or_no_convert(s)
+    if ret == None:
+        raise ValueError, "A yes or no response is required"
+    return ret
 
 def prompt_for_yes_or_no(warning, question):
     """catches yes_or_no errors and ensures a valid bool return"""
@@ -458,6 +464,7 @@ def disk_prompt(prompt_txt, arg_dict, warn_overwrite=False, prompt_size=True,
         else:
             path = None
         arg_dict["path"] = path
+        path_exists = VirtualDisk.path_exists(conn, path)
 
         sizeerr = _("A size must be specified for non-existent disks.")
         if path and not size and prompt_size:
@@ -465,7 +472,7 @@ def disk_prompt(prompt_txt, arg_dict, warn_overwrite=False, prompt_size=True,
                             "be (in gigabytes)?") % path
 
             try:
-                if not VirtualDisk.path_exists(conn, path):
+                if not path_exists:
                     size = prompt_loop(size_prompt, sizeerr, size, None, None,
                                        func=float)
             except Exception, e:
@@ -487,15 +494,16 @@ def disk_prompt(prompt_txt, arg_dict, warn_overwrite=False, prompt_size=True,
         askmsg = _("Do you really want to use this disk (yes or no)")
 
         # Prompt if disk file already exists and preserve mode is not used
-        if warn_overwrite and os.path.exists(dev.path):
-            msg = (_("This will overwrite the existing path '%s'!\n" %
+        if ((path_exists and dev.type == dev.TYPE_FILE and warn_overwrite) or
+            is_prompt()):
+            msg = (_("This will overwrite the existing path '%s'" %
                    dev.path))
             if not prompt_for_yes_or_no(msg, askmsg):
                 continue
 
         # Check disk conflicts
         if dev.is_conflict_disk(conn) is True:
-            msg = (_("Disk %s is already in use by another guest!\n" %
+            msg = (_("Disk %s is already in use by another guest" %
                    dev.path))
             if not prompt_for_yes_or_no(msg, askmsg):
                 continue
@@ -575,37 +583,18 @@ def get_vcpus(vcpus, check_cpu, guest, conn, image_vcpus=None):
 def get_cpuset(cpuset, mem, guest, conn):
     if cpuset and cpuset != "auto":
         guest.cpuset = cpuset
+
     elif cpuset == "auto":
-        caps = CapabilitiesParser.parse(conn.getCapabilities())
-        if caps.host.topology is None:
-            logging.debug("No topology section in caps xml. Skipping cpuset")
-            return
+        tmpset = None
+        try:
+            tmpset = Guest.generate_cpuset(conn, mem)
+        except Exception, e:
+            logging.debug("Not setting cpuset", str(e))
 
-        cells = caps.host.topology.cells
-        if len(cells) <= 1:
-            logging.debug("Capabilities only show <= 1 cell. Not NUMA capable")
-            return
+        if tmpset:
+            logging.debug("Auto cpuset is: %s" % tmpset)
+            guest.cpuset = tmpset
 
-        cell_mem = conn.getCellsFreeMemory(0, len(cells))
-        cell_id = -1
-        mem = mem * 1024
-        for i in range(len(cells)):
-            if cell_mem[i] > mem and len(cells[i].cpus) != 0:
-                # Find smallest cell that fits
-                if cell_id < 0 or cell_mem[i] < cell_mem[cell_id]:
-                    cell_id = i
-        if cell_id < 0:
-            logging.debug("Could not find any usable NUMA cell/cpu combinations. Not setting cpuset.")
-            return
-
-        # Build cpuset
-        cpustr = ""
-        for cpu in cells[cell_id].cpus:
-            if cpustr != "":
-                cpustr += ","
-            cpustr += str(cpu.id)
-        logging.debug("Auto cpuset is: %s" % cpustr)
-        guest.cpuset = cpustr
     return
 
 def get_network(net_kwargs, guest):
@@ -623,18 +612,9 @@ def set_os_variant(guest, distro_type, distro_variant):
         if (distro_variant and str(distro_variant).lower() != "none"):
             guest.set_os_variant(distro_variant)
 
-
-def parse_optstr(optstr, basedict=None):
-    """
-    Helper function for parsing opt strings of the form
-    opt1=val1,opt2=val2,...
-    'basedict' is a starting dictionary, so the caller can easily set
-    default values, etc.
-
-    Returns a dictionary of {'opt1': 'val1', 'opt2': 'val2'}
-    """
+def parse_optstr_tuples(optstr):
     optstr = str(optstr or "")
-    optdict = basedict or {}
+    optlist = []
 
     args = optstr.split(",")
     for opt in args:
@@ -645,9 +625,26 @@ def parse_optstr(optstr, basedict=None):
         opt_val = None
         if opt.count("="):
             opt_type, opt_val = opt.split("=", 1)
-            optdict[opt_type.lower()] = opt_val.lower()
+            optlist.append((opt_type.lower(), opt_val.lower()))
         else:
-            optdict[opt.lower()] = None
+            optlist.append((opt.lower(), None))
+
+    return optlist
+
+def parse_optstr(optstr, basedict=None):
+    """
+    Helper function for parsing opt strings of the form
+    opt1=val1,opt2=val2,...
+    'basedict' is a starting dictionary, so the caller can easily set
+    default values, etc.
+
+    Returns a dictionary of {'opt1': 'val1', 'opt2': 'val2'}
+    """
+    optlist = parse_optstr_tuples(optstr)
+    optdict = basedict or {}
+
+    for opt, val in optlist:
+        optdict[opt] = val
 
     return optdict
 
