@@ -260,6 +260,12 @@ OS_TYPES = {
                         NET  : VIRTIO_NET,
                         INPUT: USB_TABLET,
                      }},
+        "fedora14": { "label": "Fedora 14", "distro": "fedora",
+                      "devices" : {
+                        DISK : VIRTIO_DISK,
+                        NET  : VIRTIO_NET,
+                        INPUT: USB_TABLET,
+                     }},
 
         "sles10": { "label": "Suse Linux Enterprise Server",
                     "distro": "suse" },
@@ -324,6 +330,12 @@ OS_TYPES = {
                             DISK : VIRTIO_DISK,
                             NET  : VIRTIO_NET,
                         }},
+        "ubuntulucid": { "label": "Ubuntu 10.4 (Lucid Lynx)",
+                         "distro": "ubuntu",
+                         "devices" : {
+                            DISK : VIRTIO_DISK,
+                            NET  : VIRTIO_NET,
+                        }},
         "generic24": { "label": "Generic 2.4.x kernel" },
         "generic26": { "label": "Generic 2.6.x kernel" },
         "virtio26": { "sortby": "genericvirtio26",
@@ -391,16 +403,20 @@ OS_TYPES = {
 "unix": {
     "label": "UNIX",
     "variants": {
-        "freebsd6": { "label": "Free BSD 6.x" ,
+        "freebsd6": { "label": "FreeBSD 6.x" ,
                       # http://www.nabble.com/Re%3A-Qemu%3A-bridging-on-FreeBSD-7.0-STABLE-p15919603.html
                       "devices" : {
                         NET : { "model" : [ (HV_ALL, "ne2k_pci") ] }
                       }},
-        "freebsd7": { "label": "Free BSD 7.x" ,
+        "freebsd7": { "label": "FreeBSD 7.x" ,
                       "devices" : {
                         NET : { "model" : [ (HV_ALL, "ne2k_pci") ] }
                       }},
-        "openbsd4": { "label": "Open BSD 4.x" ,
+        "freebsd8": { "label": "FreeBSD 8.x" ,
+                      "devices" : {
+                        NET : { "model" : [ (HV_ALL, "e1000") ] }
+                      }},
+        "openbsd4": { "label": "OpenBSD 4.x" ,
                       # http://calamari.reverse-dns.net:980/cgi-bin/moin.cgi/OpenbsdOnQemu
                       # https://www.redhat.com/archives/et-mgmt-tools/2008-June/msg00018.html
                       "devices" : {
