@@ -38,6 +38,7 @@ SUPPORT_CONN_INTERFACE = 8
 SUPPORT_DOMAIN_GETVCPUS = 1000
 SUPPORT_DOMAIN_XML_SECURE = 1001
 SUPPORT_DOMAIN_XML_INACTIVE = 1002
+SUPPORT_DOMAIN_MANAGED_SAVE = 1003
 
 # Flags for check_pool_support
 SUPPORT_STORAGE_CREATEVOLFROM = 2000
@@ -96,7 +97,7 @@ _support_dict = {
     },
 
     SUPPORT_CONN_FINDPOOLSOURCES : {
-        "function" : "virConnect.findPoolSources",
+        "function" : "virConnect.findStoragePoolSources",
     },
 
     SUPPORT_CONN_KEYMAP_AUTODETECT : {
@@ -142,6 +143,11 @@ _support_dict = {
         "flag" : "VIR_DOMAIN_XML_SECURE",
     },
 
+    SUPPORT_DOMAIN_MANAGED_SAVE : {
+        "function" : "virDomain.hasManagedSaveImage",
+        "args" : (0,),
+    },
+
     # Pool checks
     # This can't ever require a pool object for back compat reasons
     SUPPORT_STORAGE_CREATEVOLFROM : {
